import { Algorithm } from './algorithm.abstract';
import { MatchQuery } from './match.query';
import { PassengerOrientedWaypointsCompleter } from './completer/passenger-oriented-waypoints.completer';
import { PassengerOrientedGeoFilter } from './filter/passenger-oriented-geo.filter';
import { AdRepositoryPort } from '../../ports/ad.repository.port';
import { Role } from '@modules/ad/core/domain/ad.types';
import { MatchEntity } from '@modules/ad/core/domain/match.entity';
import { Candidate } from '../../types/algorithm.types';

export class PassengerOrientedAlgorithm extends Algorithm {
  constructor(
    protected readonly query: MatchQuery,
    protected readonly repository: AdRepositoryPort,
  ) {
    super(query, repository);
    this.processors = [
      new PassengerOrientedWaypointsCompleter(),
      new PassengerOrientedGeoFilter(),
    ];
    this.candidates = [
      {
        ad: {
          id: 'cc260669-1c6d-441f-80a5-19cd59afb777',
        },
        role: Role.DRIVER,
      },
    ];
  }
  // this.candidates = (
  //   await Promise.all(
  //     sqlQueries.map(
  //       async (queryRole: QueryRole) =>
  //         ({
  //           ads: (await this.repository.queryRawUnsafe(
  //             queryRole.query,
  //           )) as AdEntity[],
  //           role: queryRole.role,
  //         } as AdsRole),
  //     ),
  //   )
  // )
  //   .map((adsRole: AdsRole) =>
  //     adsRole.ads.map(
  //       (adEntity: AdEntity) =>
  //         <Candidate>{
  //           ad: {
  //             id: adEntity.id,
  //           },
  //           role: adsRole.role,
  //         },
  //     ),
  //   )
  //   .flat();

  match = async (): Promise<MatchEntity[]> => {
    this.candidates = await this.repository.getCandidates(this.query);
    for (const processor of this.processors) {
      this.candidates = await processor.execute(this.candidates);
    }
    return this.candidates.map((candidate: Candidate) =>
      MatchEntity.create({ adId: candidate.ad.id }),
    );
  };
}
